@@ -32,7 +32,6 @@ IOCPARM_MASK     = 0x1fff
 IOC_VOID         = 0x20000000
 IOC_OUT          = 0x40000000
 IOC_IN           = 0x80000000
-
 IOC_INOUT        = IOC_IN | IOC_OUT
 
 def _IOC(inout, group, num, len):
@@ -118,7 +117,7 @@ class _PFTrans(object):
 
         for a, t in zip(self.array, trans_type):
             a.type = t
-            a.anchor = path
+            a.anchor = path.encode()
 
         self._pt = pfioc_trans(size=self.size, esize=sizeof(pfioc_trans_e),
                                array=addressof(self.array))
@@ -275,13 +274,13 @@ class PacketFilter(object):
 
     def set_optimization(self, opt="normal"):
         """Set the optimization profile for state handling like pfctl."""
-        for name, val in pf_hints[opt].iteritems():
+        for name, val in pf_hints[opt].items():
             self.set_timeout(name, val)
 
     def get_optimization(self):
         """ """
         tm = self.get_timeout()
-        for name, val in pf_hints.iteritems():
+        for name, val in pf_hints.items():
             if val["tcp.first"] == tm["tcp.first"]:
                 return name
 
@@ -291,7 +290,8 @@ class PacketFilter(object):
         Return a tuple of PFIface objects or a single PFIface object if a
         specific 'ifname' is specified.
         """
-        pi = pfioc_iface(pfiio_name=ifname, pfiio_esize=sizeof(pfi_kif))
+        pi = pfioc_iface(pfiio_name=ifname.encode(),
+                         pfiio_esize=sizeof(pfi_kif))
 
         with open(self.dev, 'w') as d:
             ioctl(d, DIOCIGETIFACES, pi)
@@ -306,7 +306,7 @@ class PacketFilter(object):
 
     def set_ifflags(self, ifname, flags):
         """Set the user setable 'flags' on the interface 'ifname'."""
-        pi = pfioc_iface(pfiio_name=ifname, pfiio_flags=flags)
+        pi = pfioc_iface(pfiio_name=ifname.encode(), pfiio_flags=flags)
         with open(self.dev, 'w') as d:
             with _PFTrans(d):
                 ioctl(d, DIOCSETIFFLAG, pi)
@@ -319,7 +319,7 @@ class PacketFilter(object):
         if flags is None:
             flags = PFI_IFLAG_SKIP
 
-        pi = pfioc_iface(pfiio_name=ifname, pfiio_flags=flags)
+        pi = pfioc_iface(pfiio_name=ifname.encode(), pfiio_flags=flags)
         with open(self.dev, 'w') as d:
             with _PFTrans(d):
                 ioctl(d, DIOCCLRIFFLAG, pi)
@@ -330,7 +330,7 @@ class PacketFilter(object):
         If no 'ifname' is provided, turn off the collection of per-interface
         statistics. Raise PFError if 'ifname' is not a valid interface name.
         """
-        pi = pfioc_iface(pfiio_name=ifname)
+        pi = pfioc_iface(pfiio_name=ifname.encode())
         with open(self.dev, 'w') as d:
             with _PFTrans(d):
                 try:
@@ -354,7 +354,7 @@ class PacketFilter(object):
         An optional 'ifname' can be specified in order to clear statistics only
         for a specific interface.
         """
-        pi = pfioc_iface(pfiio_name=ifname)
+        pi = pfioc_iface(pfiio_name=ifname.encode())
         with open(self.dev, 'w') as d:
             ioctl(d, DIOCCLRSTATUS, pi)
 
@@ -459,7 +459,7 @@ class PacketFilter(object):
         if path.endswith("/*"):
             path = path[:-2]
 
-        pr = pfioc_rule(anchor=path)
+        pr = pfioc_rule(anchor=path.encode())
         if clear:
             pr.action = PF_GET_CLR_CNTR
 
@@ -472,8 +472,8 @@ class PacketFilter(object):
             pr.nr = nr
             ioctl(dev, DIOCGETRULE, pr)
             if pr.anchor_call:
-                path = os.path.join(pr.anchor, pr.anchor_call)
-                rs = PFRuleset(pr.anchor_call, pr.rule)
+                path = os.path.join(pr.anchor.decode(), pr.anchor_call.decode())
+                rs = PFRuleset(pr.anchor_call.decode(), pr.rule)
                 rs.append(*self._get_rules(path, dev, clear))
                 rules.append(rs)
             else:
@@ -494,7 +494,7 @@ class PacketFilter(object):
             for rule in self._get_rules(path, d, clear):
                 if isinstance(rule, PFRule):
                     if not all((getattr(rule, attr) == value)
-                               for (attr, value) in kw.iteritems()):
+                               for (attr, value) in kw.items()):
                         continue
                 rs.append(rule)
         return rs
@@ -532,11 +532,12 @@ class PacketFilter(object):
                             self._inadefine(t, d, path, a.ticket)
                     elif a.type == PF_TRANS_RULESET:
                         for r in ruleset.rules:
-                            pr = pfioc_rule(ticket=a.ticket, anchor=path,
+                            pr = pfioc_rule(ticket=a.ticket,
+                                            anchor=path.encode(),
                                             rule=r._to_struct())
 
                             if isinstance(r, PFRuleset):
-                                pr.anchor_call = r.name
+                                pr.anchor_call = r.name.encode()
 
                             ioctl(d, DIOCADDRULE, pr)
 
@@ -587,8 +588,8 @@ class PacketFilter(object):
 
         buffer = (pfr_table * len(tables))()
         for (t, b) in zip(tables, buffer):
-            b.pfrt_name = t.name
-            b.pfrt_anchor = t.anchor
+            b.pfrt_name = t.name.encode()
+            b.pfrt_anchor = t.anchor.encode()
 
         io.pfrio_buffer = addressof(buffer)
 
@@ -607,7 +608,7 @@ class PacketFilter(object):
         io = pfioc_table(pfrio_esize=sizeof(pfr_table))
 
         if filter is not None:
-            io.pfrio_table = pfr_table(pfrt_anchor=filter.anchor)
+            io.pfrio_table = pfr_table(pfrt_anchor=filter.anchor.encode())
 
         with open(self.dev, 'w') as d:
             while True:
@@ -639,10 +640,11 @@ class PacketFilter(object):
         table name; 'addrs' can be either PFTableAddr instances or strings.
         Return the addresses that match.
         """
-        if isinstance(table, basestring):
-            table = pfr_table(pfrt_name=table)
+        if isinstance(table, str):
+            table = pfr_table(pfrt_name=table.encode())
         else:
-            table = pfr_table(pfrt_name=table.name, pfrt_anchor=table.anchor)
+            table = pfr_table(pfrt_name=table.name.encode(),
+                              pfrt_anchor=table.anchor.encode())
 
         _addrs = []
         for addr in addrs:
@@ -669,10 +671,11 @@ class PacketFilter(object):
         table name; 'addrs' can be either PFTableAddr instances or strings.
         Return the number of addresses effectively added.
         """
-        if isinstance(table, basestring):
-            table = pfr_table(pfrt_name=table)
+        if isinstance(table, str):
+            table = pfr_table(pfrt_name=table.encode())
         else:
-            table = pfr_table(pfrt_name=table.name, pfrt_anchor=table.anchor)
+            table = pfr_table(pfrt_name=table.name.encode(),
+                              pfrt_anchor=table.anchor.encode())
 
         _addrs = []
         for addr in addrs:
@@ -697,10 +700,11 @@ class PacketFilter(object):
 
         Return the number of addresses removed.
         """
-        if isinstance(table, basestring):
+        if isinstance(table, str):
             table = pfr_table(pfrt_name=table)
         else:
-            table = pfr_table(pfrt_name=table.name, pfrt_anchor=table.anchor)
+            table = pfr_table(pfrt_name=table.name.encode(),
+                              pfrt_anchor=table.anchor.encode())
 
         io = pfioc_table(pfrio_table=table)
 
@@ -716,10 +720,11 @@ class PacketFilter(object):
         table name; 'addrs' can be either PFTableAddr instances or strings.
         Return the number of addresses deleted.
         """
-        if isinstance(table, basestring):
+        if isinstance(table, str):
             table = pfr_table(pfrt_name=table)
         else:
-            table = pfr_table(pfrt_name=table.name, pfrt_anchor=table.anchor)
+            table = pfr_table(pfrt_name=table.name.encode(),
+                              pfrt_anchor=table.anchor.encode())
 
         _addrs = []
         for addr in addrs:
@@ -747,10 +752,11 @@ class PacketFilter(object):
         Return a tuple containing the number of addresses deleted, added and
         changed.
         """
-        if isinstance(table, basestring):
+        if isinstance(table, str):
             table = pfr_table(pfrt_name=table)
         else:
-            table = pfr_table(pfrt_name=table.name, pfrt_anchor=table.anchor)
+            table = pfr_table(pfrt_name=table.name.encode(),
+                              pfrt_anchor=table.anchor.encode())
 
         _addrs = []
         for addr in addrs:
@@ -776,10 +782,11 @@ class PacketFilter(object):
         'table' can be either a PFTable instance or a string containing the
         table name. Return a list of PFTableAddr objects.
         """
-        if isinstance(table, basestring):
-            table = pfr_table(pfrt_name=table)
+        if isinstance(table, str):
+            table = pfr_table(pfrt_name=table.encode())
         else:
-            table = pfr_table(pfrt_name=table.name, pfrt_anchor=table.anchor)
+            table = pfr_table(pfrt_name=table.name.encode(),
+                              pfrt_anchor=table.anchor.encode())
 
         io = pfioc_table(pfrio_table=table, pfrio_esize=sizeof(pfr_addr))
 
@@ -837,8 +844,8 @@ class PacketFilter(object):
 
         buffer = (pfr_table * len(tables))()
         for (t, b) in zip(tables, buffer):
-            b.pfrt_name = t.name
-            b.pfrt_anchor = t.anchor
+            b.pfrt_name = t.name.encode()
+            b.pfrt_anchor = t.anchor.encode()
 
         io.pfrio_buffer = addressof(buffer)
 
