@@ -29,7 +29,7 @@ __all__ = ['PFUid',
 # Helper functions
 def azero(seq):
     """Return True if all numbers in 'seq' are 0s."""
-    return not filter(None, seq)
+    return all(v == 0 for v in seq)
 
 
 class PFOp(PFObject):
@@ -39,7 +39,7 @@ class PFOp(PFObject):
         """Check arguments and initialize instance attributes."""
         self.op = op
 
-        if isinstance(num, basestring) or isinstance(num, Structure):
+        if isinstance(num, str) or isinstance(num, Structure):
             super(PFOp, self).__init__(num)
         elif num is None:
             self.num = (0, 0)
@@ -216,7 +216,7 @@ class PFAddr(PFObject):
         self.af = af
 
         if addr is None:
-            t = (kw["type"] if kw.has_key("type") else PF_ADDR_ADDRMASK)
+            t = kw.get("type", PF_ADDR_ADDRMASK)
             addr = pf_addr_wrap(type=t)
 
         super(PFAddr, self).__init__(addr, **kw)
@@ -240,19 +240,19 @@ class PFAddr(PFObject):
             addr = mask = None
 
         if self.type == PF_ADDR_DYNIFTL:
-            self.ifname = a.v.ifname
+            self.ifname = a.v.ifname.decode()
             self.iflags = a.iflags
             self.dyncnt = a.p.dyncnt
             self.mask = mask
         elif self.type == PF_ADDR_TABLE:
-            self.tblname = a.v.tblname
+            self.tblname = a.v.tblname.decode()
             self.tblcnt = a.p.tblcnt
         elif self.type == PF_ADDR_ADDRMASK:
             self.addr = addr
             self.mask = mask
         elif self.type == PF_ADDR_RTLABEL:
-            self.rtlabelname = a.v.rtlabelname
-            self.rtlabel = a.v.rtlabel
+            self.rtlabelname = a.v.rtlabelname.decode()
+            self.rtlabel = a.v.rtlabel.decode()
         elif self.type == PF_ADDR_RANGE:
             self.addr = (addr, mask)
 
@@ -341,16 +341,16 @@ class PFAddr(PFObject):
         a.type = self.type
 
         if self.type == PF_ADDR_DYNIFTL:
-            a.v.ifname = self.ifname
+            a.v.ifname = self.ifname.encode()
             a.p.dyncnt = self.dyncnt
             a.iflags = self.iflags
             if self.af == AF_UNSPEC:
-                mask = '\xff' * 16
+                mask = b'\xff' * 16
             else:
                 mask = inet_pton(self.af, self.mask)
             memmove(a.v.a.mask.v6, c_char_p(mask), len(mask))
         elif self.type == PF_ADDR_TABLE:
-            a.v.tblname = self.tblname
+            a.v.tblname = self.tblname.encode()
             a.p.tblcnt = self.tblcnt
         elif self.type == PF_ADDR_ADDRMASK and self.addr:
             addr = inet_pton(self.af, self.addr)
@@ -358,7 +358,7 @@ class PFAddr(PFObject):
             memmove(a.v.a.addr.v6, c_char_p(addr), len(addr))
             memmove(a.v.a.mask.v6, c_char_p(mask), len(mask))
         elif self.type == PF_ADDR_RTLABEL:
-            a.v.rtlabelname = self.rtlabelname
+            a.v.rtlabelname = self.rtlabelname.encode()
             a.v.rtlabel = self.rtlabel
         elif self.type == PF_ADDR_RANGE:
             addr1 = inet_pton(self.af, self.addr[0])
@@ -421,7 +421,7 @@ class PFAddr(PFObject):
             return (self.addr == a.addr and
                     self.mask == a.mask)
         elif self.type == PF_ADDR_RTLABEL:
-            return (self.rtlabelname == a.rtlabelname)
+            return (self.rtlabelname == a.rtlabelname.decode())
         elif self.type == PF_ADDR_RANGE:
             return (self.addr == a.addr)
 
@@ -505,7 +505,7 @@ class PFPool(PFObject):
         self.addr       = PFAddr(p.addr, self._af)
         self.key        = "{:#010x}{:08x}{:08x}{:08x}".format(*p.key.key32)
         self.counter    = p.counter
-        self.ifname     = p.ifname
+        self.ifname     = p.ifname.decode()
         self.tblidx     = p.tblidx
         self.states     = p.states
         self.curweight  = p.curweight
@@ -521,7 +521,7 @@ class PFPool(PFObject):
             p.addr = PFAddr(type=PF_ADDR_NONE)._to_struct()
         else:
             p.addr       = self.addr._to_struct()
-        p.ifname     = self.ifname
+        p.ifname     = self.ifname.encode()
         p.states     = self.states
         p.curweight  = self.curweight
         p.weight     = self.weight
@@ -684,8 +684,8 @@ class PFRule(PFObject):
         self.dst               = PFRuleAddr(r.dst, af=r.af, proto=r.proto)
         #skip
         self.label             = r.label
-        self.ifname            = r.ifname
-        self.rcv_ifname        = r.rcv_ifname
+        self.ifname            = r.ifname.decode()
+        self.rcv_ifname        = r.rcv_ifname.decode()
         self.qname             = r.qname
         self.pqname            = r.pqname
         self.tagname           = r.tagname
@@ -733,6 +733,7 @@ class PFRule(PFObject):
         self.tag               = r.tag
         self.match_tag         = r.match_tag
         self.scrub_flags       = r.scrub_flags
+        self.delay             = r.delay
         self.uid               = PFUid(r.uid)
         self.gid               = PFGid(r.gid)
         self.rule_flag         = r.rule_flag
@@ -774,8 +775,8 @@ class PFRule(PFObject):
         r.dst               = self.dst._to_struct()
         #skip
         r.label             = self.label
-        r.ifname            = self.ifname
-        r.rcv_ifname        = self.rcv_ifname
+        r.ifname            = self.ifname.encode()
+        r.rcv_ifname        = self.rcv_ifname.encode()
         r.qname             = self.qname
         r.pqname            = self.pqname
         r.tagname           = self.tagname
@@ -814,6 +815,8 @@ class PFRule(PFObject):
         r.max_mss           = self.max_mss
         r.tag               = self.tag
         r.match_tag         = self.match_tag
+        r.scrub_flags       = self.scrub_flags
+        r.delay             = self.delay
         r.uid               = self.uid._to_struct()
         r.gid               = self.gid._to_struct()
         r.rule_flag         = self.rule_flag
@@ -974,15 +977,23 @@ class PFRule(PFObject):
         if self.pktrate.limit:
             s += " {.pktrate}".format(self)
 
-        if self.scrub_flags & PFSTATE_SETMASK:
+        if self.scrub_flags & PFSTATE_SETMASK or self.qname or \
+           self.rule_flag & PFRULE_SETDELAY:
             opts = []
             if self.scrub_flags[0] != PFSTATE_SETPRIO:
                 if self.set_prio[0] == self.set_prio[1]:
                     opts.append("prio {}".format(self.set_prio[0]))
                 else:
                     opts.append("prio ({}, {})".format(*self.set_prio))
+            if self.qname:
+                if self.pqname:
+                    opts.append("queue ({.qname}, {.pqname})".format(self))
+                else:
+                    opts.append("queue {.qname}".format(self))
             if self.scrub_flags & PFSTATE_SETTOS:
                 opts.append("tos {.set_tos:#04x}".format(self))
+            if self.rule_flag & PFRULE_SETDELAY:
+                opts.append("delay {.delay}".format(self))
             s += " set ( {} )".format(", ".join(opts))
 
         has_opts = False
@@ -1040,7 +1051,7 @@ class PFRule(PFObject):
                 opts.append("pflow")
             for i, t in enumerate(self.timeout):
                 if t:
-                    tm = [k for (k, v) in pf_timeouts.iteritems() if v == i][0]
+                    tm = [k for (k, v) in pf_timeouts.items() if v == i][0]
                     opts.append("{} {}".format(tm, t))
 
             s += " ({})".format(", ".join(opts))
